
/***********************************************************************************/
/* Copyright (c) 2012, SAS Institute Inc., Cary, NC, USA, All Rights Reserved      */
/*                                                                                 */
/* NAME:          ecmrr_efile_fincen_newctr_inst                                   */
/* PRODUCT:       Enterprise Case Management                                       */
/* PURPOSE:       dedupped the institution involved for CTR efiling and assign     */
/*                institution seq number                                           */
/* PARAMETERS:                                                                     */
/*                in_dsn: input_institution table name                             */
/*                               rr_rk                                             */
/*                out_dsn: output table name                                       */
/*                seq_dsn: sequence table to be updated with Z_INST_SEQ_NO         */
/* NOTE:          All ctr reports with the same set of insttitutions should        */
/*                be reported under the same 2B                                    */
/*                institution involved is identified by                            */ 
/*                    X_INSTITUTION.X_INSTITUTION_RK                               */
/*                There may be multiple X_INSTITUTION_RK for a RR_RK               */ 
/*                                                                                 */
/***********************************************************************************/



%macro ecmrr_efile_fincen_newctr_inst(in_dsn=r_x_institution
                                      ,out_dsn=t_z_x_institution
                                      ,seq_dsn=t_z_seq_no );
                                          
%put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;


/* Assign the known seq numbers to each report in the institution table */ 
proc sql noprint;
     create table t_joined as
     select a.*
           , b.z_batch_seq_no
           , b.z_filing_inst_seq_no          
     from &in_dsn as a,
          &seq_dsn as b
     where a.rr_rk=b.rr_rk;
quit; 

        
%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;

/* Order the reports in the input data by rr_rk and institution_rk */
/* NODUPKEY is added in case there are duplicated involved institutions within a report */
proc sort data=t_joined out=t_sorted nodupkey ;
     by z_batch_seq_no z_filing_inst_seq_no rr_rk x_institution_rk;
run;
        
%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;

/* transpose the list of institutions to a single field for each report */

data t_transposed;
     set t_sorted;
     by z_batch_seq_no z_filing_inst_seq_no rr_rk;
     length z_institution_rk_list $ 4096;
     retain z_institution_rk_list '';
     if first.rr_rk then z_institution_rk_list='';
     z_institution_rk_list=left(strip(z_institution_rk_list)
                           ||' '||strip(put(x_institution_rk,best.)));
     if last.rr_rk then output;
     keep z_batch_seq_no z_filing_inst_seq_no rr_rk z_institution_rk_list;
     
run;

        
%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end; 
    
/* keep one occurence of institution detail for each batch + filing institution + institution  */
proc sort data=t_joined out=t_discrete_inst nodupkey;
     by z_batch_seq_no z_filing_inst_seq_no x_institution_rk;
run;
        
%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;

/* Get the unique list of 2B and assign Z_INST_SEQ_NO to each record */

proc sql noprint;

     create table t_z_inst_seq_no as
     select distinct  z_batch_seq_no
                     ,z_filing_inst_seq_no
                     ,z_institution_rk_list
                     ,0 as z_inst_seq_no
                    
     from t_transposed
;
 
     update t_z_inst_seq_no 
            set z_inst_seq_no=monotonic()
     ;

        
%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;

/* expand the list of 2B into institution level */
data t_z_inst_expanded  
     ;
    set t_z_inst_seq_no;
    i_inst=1;
    do until (scan(z_institution_rk_list,i_inst,' ') = '');
       x_institution_rk=input(scan(z_institution_rk_list,i_inst,' '),best.);
       output t_z_inst_expanded;
       i_inst+1;
    end;
run;

        
%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;

/* Merge 2B at institution level with institution details */
proc sql;
     create table &out_dsn as
     select a.*
            , 0 as z_rr_seq_no
            , b.z_inst_seq_no
            , 0 as z_subject_seq_no
            ,'2B' as form_efile_record_id 
     from t_discrete_inst (drop=rr_rk) as a,
          t_z_inst_expanded as b
     where a.x_institution_rk=b.x_institution_rk 
           and a.z_batch_seq_no=b.z_batch_seq_no 
           and a.z_filing_inst_seq_no=b.z_filing_inst_seq_no
    order by z_batch_seq_no, z_filing_inst_seq_no, 
             z_inst_seq_no, z_rr_seq_no, z_subject_seq_no, row_no

;
     /* update the seq table with z_inst_seq_no */
     update &seq_dsn as c
     set z_inst_seq_no=(select d.z_inst_seq_no 
                        from (select distinct a.z_batch_seq_no
                                              ,a.z_filing_inst_seq_no
                                              ,a.rr_rk
                                              , b.z_inst_seq_no 
                              from t_transposed as a,
                                   t_z_inst_seq_no as b
                               where a.z_batch_seq_no=b.z_batch_seq_no
                                     and a.z_filing_inst_seq_no=b.z_filing_inst_seq_no
                                     and a.z_institution_rk_list=b.z_institution_rk_list
                              ) d
                        where c.rr_rk=d.rr_rk)
     
      ;
    /* If there is no x_institution, assign 0 to z_inst_seq_no */  
     update &seq_dsn 
     set z_inst_seq_no=0
     where z_inst_seq_no=.
     
      ;
quit;  
        
%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;
 
%mExit:

%put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;

%mend;
 
