
/***********************************************************************************/
/* Copyright (c) 2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved      */
/*                                                                                 */
/* NAME:          ecmrr_efile_fincen_newctr_sum                                    */
/* PRODUCT:       Enterprise Case Management                                       */
/* PURPOSE:       summarize the detail data prepared by                            */
/*                ecmrr_efile_fincen_newctr_detail and assign sequence number for  */
/*                batch e-filing                                                   */                                                
/* PARAMETERS:                                                                     */
/*                form_config_rk: form config key                                  */
/* NOTE:          Input tables:                                                    */
/*                  x_transmitter             (1A)                                 */  
/*                  t_zx_institution          (2B)                                 */  
/*                  t_zx_rr                   (3A 2A 9B)                           */ 
/*                  r_x_foreign_cash_in       (3B)                                 */
/*                  r_x_foreign_cash_out      (3C)                                 */
/*                  t_zx_subject              (4A)                                 */
/*                  r_x_account_cash_in       (4B)                                 */
/*                  r_x_account_cash_out      (4C)                                 */
/*                Output tables:                                                   */
/*                  zx_transmitter            (1A)                                 */  
/*                  zx_filing_inst            (2A)                                 */
/*                  zx_institution            (2B)                                 */  
/*                  zx_rr                     (3A)                                 */ 
/*                  zx_foregin_cash_in        (3B)                                 */
/*                  zx_foreign_cash_out       (3C)                                 */
/*                  zx_person_involved        (4A)                                 */
/*                  zx_account_cash_in        (4B)                                 */
/*                  zx_account_cash_out       (4C)                                 */
/*                  z_sum_inst                (9A)                                 */
/*                  z_sum_filing_inst         (9B)                                 */
/*                  z_sum_transmitter         (9Z)                                 */
/***********************************************************************************/

%macro ecmrr_efile_fincen_newctr_sum(form_config_rk=202);

%put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;

%let successYN=Y;
/* get the max number of reports can be added to the efile for this form type */
%let efile_max_rr_cnt=0;

proc sql noprint;
     select efile_max_rr_cnt into : efile_max_rr_cnt
     from ecm_db.form_config
     where form_config_rk=&form_config_rk;

%if &efile_max_rr_cnt < 1 %then %let efile_max_rr_cnt=700; 

/*****************************************************************************/
/* create sequence numbers by batch, filing institution, and rr              */
/* institution seq number will be assigned later                             */
/*****************************************************************************/
                    
/* sort reports by filing institution and report rk */
   proc sort data=t_zx_rr (keep=x_institution_rk rr_rk) 
             out=key_sorted;
        by x_institution_rk rr_rk;
        
   run;

%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;



/* create z_batch_seq_no, z_filing_inst_seq_no, and z_rr_seq_no */
   data t_z_seq_no  ;
        set key_sorted;
        by x_institution_rk rr_rk; 

        retain z_batch_seq_no 0;
        retain z_filing_inst_seq_no 0;
        retain z_inst_seq_no        0; /* to be assinged later */

        /* rr_seq_no is unique within a batch, not within a filing institution */
        
        if mod(_n_,&efile_max_rr_cnt)=1 then
        do;
           z_batch_seq_no+1;
           if first.x_institution_rk then z_filing_inst_seq_no=0;
           else z_filing_inst_seq_no=1;
           z_rr_seq_no=0;
        end;
        if first.x_institution_rk then
        do;
                z_filing_inst_seq_no+1;
        end;
         
        z_rr_seq_no+1;
        
       
    run;

/****************************************************/
/* Create 2B records  and assign z_inst_seq_no      */
/* to t_z_seq_no                                    */
/****************************************************/ 

%ecmrr_efile_fincen_newctr_inst(in_dsn=r_x_institution
                                ,seq_dsn=t_z_seq_no
                                ,out_dsn=zx_institution
                                );

%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;
         
/****************************************************/
/* assign RR seq number to RR level tables          */
/****************************************************/ 
%ecmrr_efile_fincen_newctr_seqno(in_dsn=t_zx_rr
                                 , out_dsn=zx_rr
                                 , seq_dsn=t_z_seq_no
                                 , z_subject_seq_no=0
                                 , row_no=1
                                 , form_efile_record_id=3A 
                                 );   
%if &successYN=N %then %goto mExit;

%ecmrr_efile_fincen_newctr_seqno(in_dsn=r_x_foreign_cash_in
                                 , out_dsn=zx_foreign_cash_in
                                 , seq_dsn=t_z_seq_no
                                 , z_subject_seq_no=0
                                 , row_no=
                                 , form_efile_record_id=3B 
                                 );   
%if &successYN=N %then %goto mExit;


%ecmrr_efile_fincen_newctr_seqno(in_dsn=r_x_foreign_cash_out
                                 , out_dsn=zx_foreign_cash_out
                                 , seq_dsn=t_z_seq_no
                                 , z_subject_seq_no=0
                                 , row_no= 
                                 , form_efile_record_id=3C 
                                 );   
%if &successYN=N %then %goto mExit;



%ecmrr_efile_fincen_newctr_seqno(in_dsn=t_zx_person_involved
                                 , out_dsn=zx_person_involved
                                 , seq_dsn=t_z_seq_no
                                 , z_subject_seq_no=
                                 , row_no=0
                                 , form_efile_record_id=4A  
                                 );   
%if &successYN=N %then %goto mExit;



%ecmrr_efile_fincen_newctr_seqno(in_dsn=t_zx_account_cash_in
                                 , out_dsn=zx_account_cash_in
                                 , seq_dsn=t_z_seq_no
                                 , z_subject_seq_no=
                                 , row_no= 
                                 , form_efile_record_id=4B 
                                 );

%if &successYN=N %then %goto mExit;

%ecmrr_efile_fincen_newctr_seqno(in_dsn=t_zx_account_cash_out
                                 , out_dsn=zx_account_cash_out
                                 , seq_dsn=t_z_seq_no
                                 , z_subject_seq_no=
                                 , row_no= 
                                 , form_efile_record_id=4C 
                                 );   
%if &successYN=N %then %goto mExit;



/********************************************/
/* Create z_filing_inst (2A)                */
/********************************************/ 
/* remove duplicated filing institution information */ 
proc sort data=zx_rr (keep= z_batch_seq_no
                            z_filing_inst_seq_no
                            z_inst_seq_no
                            z_rr_seq_no
                            z_subject_seq_no
                            form_efile_record_id
                            X_FILING_INST_PRIM_FED_REG_CD
                            X_FILING_INST_NM
                            X_FILING_INST_TIN_NO
                            X_FILING_INST_ADDRESS_TXT 
                            X_FILING_INST_CITY_NM 
                            X_FILING_INST_STATE_CD 
                            X_FILING_INST_POSTAL_CD 
                           
                        )
          out=zx_filing_inst 
          nodupkey;
     by z_batch_seq_no z_filing_inst_seq_no;
run;

%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;

/* set record id to 2A */
proc sql noprint;
     /* force rr and subject sequence to be the highest so that 9B record will be after all the 3A records */
     update zx_filing_inst 
     set form_efile_record_id='2A',
         z_inst_seq_no=0,
         z_rr_seq_no=0
    ; 
       
quit;

 
%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;

 

/********************************************/
/* Create z_sum_inst (9A)                   */
/********************************************/ 
proc summary data=zx_rr nway missing;
          class z_batch_seq_no z_filing_inst_seq_no z_inst_seq_no; 
          id z_rr_seq_no row_no z_subject_seq_no form_efile_record_id;        
          var z_subject_cnt  z_cash_in_acct_cnt z_cash_out_acct_cnt;
          output out=z_sum_inst (drop=_type_ _freq_) sum= n=z_rr_cnt;
          
run;


 
%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;

proc sql noprint;
     /* force rr and subject sequence to be the highest so that 9B record will be after all the 3A records */
     update z_sum_inst 
     set z_rr_seq_no=99999,
         z_subject_seq_no=99999,
         form_efile_record_id='9A'
    ; 
       
quit;

%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;
   
 

/********************************************/
/* Create z_sum_filing_inst (9B)            */
/********************************************/ 
proc summary data=z_sum_inst nway missing;
          class z_batch_seq_no z_filing_inst_seq_no; 
          id z_rr_seq_no z_inst_seq_no z_subject_seq_no form_efile_record_id;
          var z_rr_cnt z_subject_cnt z_cash_in_acct_cnt z_cash_out_acct_cnt;
          output out=z_sum_filing_inst (drop=_type_ _freq_) sum= n=z_inst_cnt ;
run;


 
%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;

proc sql;
     alter table z_sum_filing_inst
      add z_inst_cnt num ;
     ;

 
%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;

 
proc sql noprint;
     /* force rr and subject sequence to be the highest so that 9B record will be after all the 3A records */
     update z_sum_filing_inst as a 
     set z_rr_seq_no=99999
         , z_inst_seq_no=99999
         , z_subject_seq_no=99999
         , form_efile_record_id='9B'
         , z_inst_cnt=(select b.z_inst_cnt 
                       from 
                             (select distinct z_batch_seq_no
                                              , z_filing_inst_seq_no
                                              , count(*) as z_inst_cnt 
                               from zx_institution
                               group by z_batch_seq_no, z_filing_inst_seq_no
                              ) as b
                       where a.z_batch_seq_no=b.z_batch_seq_no 
                             and a.z_filing_inst_seq_no=b.z_filing_inst_seq_no
                       )
    ; 
       
quit;

%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;
   
 

/********************************************/
/* Create zx_transmitter (1A)               */
/********************************************/ 
proc sql noprint;
   create table zx_transmitter as
     select  a.*
             , b.z_batch_seq_no
             , 0 as z_filing_inst_seq_no
             , 0 as z_inst_seq_no
             , 0 as z_rr_seq_no
             , 0 as z_subject_seq_no
             , b.z_coverage_start_dt
             , b.z_coverage_end_dt                
             , '1A' as form_efile_record_id 
     from x_transmitter as a,
          (select min(x_transaction_dt) as z_coverage_start_dt format=yymmddn8.,
           max(x_transaction_dt) as z_coverage_end_dt format=yymmddn8.,
           z_batch_seq_no
           from zx_rr
           group by z_batch_seq_no) as b         
     ;

quit; 

%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end; 
     
/********************************************/
/* Create z_sum_transmitter  (9Z)           */
/********************************************/ 
 

proc summary data=z_sum_filing_inst missing nway;
          class z_batch_seq_no ;
          id z_filing_inst_seq_no z_rr_seq_no z_inst_seq_no z_subject_seq_no form_efile_record_id; 
          var z_inst_cnt z_rr_cnt z_subject_cnt z_cash_in_acct_cnt z_cash_out_acct_cnt ;
          output out=z_sum_transmitter (drop=_freq_ _type_) sum= n=z_filing_inst_cnt;
run;


proc sql noprint;

    update z_sum_transmitter
    set z_filing_inst_seq_no=99999,
        z_rr_seq_no=99999,
        z_inst_seq_no=99999,
        z_subject_seq_no=99999,
        form_efile_record_id='9Z'
        ;

quit;

%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;


%MEXIT: 

    %if &ecm_debug=N %then
    %do;
        proc datasets nolist nowarn lib=work;
             delete t_zx:
                    t_z_:
                    r_x:
             ;
        quit;
    %end;


%put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;


%mend;
 
